"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var paperclip_1 = require("paperclip");
var translate_utils_1 = require("./translate-utils");
var utils_1 = require("./utils");
var lodash_1 = require("lodash");
var path = require("path");
exports.compile = function (_a, filePath, options) {
    var ast = _a.ast, sheet = _a.sheet;
    if (options === void 0) { options = {}; }
    var context = translate_utils_1.createTranslateContext(filePath, paperclip_1.getImportIds(ast), paperclip_1.getPartIds(ast), paperclip_1.getStyleScopes(fs)(ast, filePath), Boolean(paperclip_1.getLogicElement(ast)), options);
    context = translateRoot(ast, sheet, context);
    return context.buffer;
};
var translateRoot = function (ast, sheet, context) {
    context = translateImports(ast, context);
    if (sheet) {
        context = translateStyleSheet(sheet, context);
    }
    var logicElement = paperclip_1.getLogicElement(ast);
    if (logicElement) {
        var src = paperclip_1.getAttributeStringValue("src", logicElement);
        if (src) {
            var logicRelativePath = paperclip_1.getRelativeFilePath(fs)(context.filePath, src);
            context = translate_utils_1.addBuffer("const logic = require(\"" + logicRelativePath + "\");\n", context);
            context = translate_utils_1.addBuffer("const enhanceView = logic.default || logic;\n\n", context);
        }
    }
    context = translateUtils(ast, context);
    context = translateParts(ast, context);
    context = translateDefaultView(ast, context);
    return context;
};
var translateStyleSheet = function (sheet, context) {
    if (!sheet.rules.length) {
        return context;
    }
    context = translate_utils_1.addBuffer("if (typeof document !== \"undefined\") {\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.addBuffer("const style = document.createElement(\"style\");\n", context);
    context = translate_utils_1.addBuffer("style.textContent = " + JSON.stringify(paperclip_1.stringifyCSSSheet(sheet, null).replace(/[\s\r\n\t]+/g, " ")) + ";\n", context);
    context = translate_utils_1.addBuffer("document.body.appendChild(style);\n", context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("}\n\n", context);
    return context;
};
var translateUtils = function (ast, context) {
    context = translateStyledUtil(ast, context);
    // KEEP ME: for logic
    // context = translateExtendsPropsUtil(ast, context);
    return context;
};
var translateStyleScopeAttributes = function (context, newLine) {
    if (newLine === void 0) { newLine = ""; }
    for (var i = 0, length_1 = context.styleScopes.length; i < length_1; i++) {
        var scope = context.styleScopes[i];
        context = translate_utils_1.addBuffer("\"data-pc-" + scope + "\": true," + newLine, context);
    }
    return context;
};
var translateExtendsPropsUtil = function (ast, context) {
    context = translate_utils_1.addBuffer("const extendProps = (defaultProps, extender) => {\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.addBuffer("return typeof extender === 'function' ? extender(defaultProps) : Object.assign(defaultProps, extender);\n", context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("};\n\n", context);
    return context;
};
var translateStyledUtil = function (ast, context) {
    context = translate_utils_1.addBuffer("export function styled(tagName, defaultProps) {\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.addBuffer("return function(props) {\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.addBuffer("return React.createElement(tagName, Object.assign({ ", context);
    context = translateStyleScopeAttributes(context, " ");
    context = translate_utils_1.addBuffer("}, defaultProps, props));\n", context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("};\n", context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("}\n\n", context);
    return context;
};
var translateImports = function (ast, context) {
    context = translate_utils_1.addBuffer("const React = require(\"react\");\n", context);
    var imports = paperclip_1.getImports(ast);
    for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
        var imp = imports_1[_i];
        var id = paperclip_1.getAttributeStringValue("id", imp);
        var src = paperclip_1.getAttributeStringValue("src", imp);
        if (!src) {
            continue;
        }
        var relativePath = path
            .relative(path.dirname(context.filePath), paperclip_1.resolveImportFile(fs)(context.filePath, src))
            .replace(/\\/g, "/");
        if (relativePath.charAt(0) !== ".") {
            relativePath = "./" + relativePath;
        }
        if (id) {
            context = translate_utils_1.addBuffer("import " + utils_1.pascalCase(id) + " from \"" + relativePath + "\";\n", context);
        }
        else {
            context = translate_utils_1.addBuffer("import \"" + relativePath + "\";\n", context);
        }
    }
    context = translate_utils_1.addBuffer("\n", context);
    return context;
};
var translateParts = function (root, context) {
    for (var _i = 0, _a = paperclip_1.getParts(root); _i < _a.length; _i++) {
        var part = _a[_i];
        if (paperclip_1.hasAttribute(paperclip_1.NO_COMPILE_TAG_NAME, part)) {
            continue;
        }
        // already compiled
        if (paperclip_1.getAttributeStringValue("id", part) === paperclip_1.DEFAULT_PART_ID) {
            continue;
        }
        context = translatePart(part, context);
    }
    return context;
};
var translatePart = function (part, context) {
    var componentName = utils_1.pascalCase(paperclip_1.getAttributeStringValue("id", part));
    context = translateComponent(componentName, part, true, context);
    return context;
};
var translateComponent = function (componentName, node, shouldExport, context) {
    context = translate_utils_1.startBlock(translate_utils_1.addBuffer((shouldExport ? "export " : "") + "const " + componentName + " = React.memo(function " + componentName + "(props) {\n", context));
    context = translate_utils_1.addBuffer("return ", context);
    context = translateJSXRoot(node, context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer(";\n", context);
    context = translate_utils_1.addBuffer("});\n\n", context);
    return context;
};
var translateDefaultView = function (root, context) {
    var target = paperclip_1.getDefaultPart(root) || root;
    var visibleChildren = paperclip_1.getVisibleChildNodes(target);
    if (!visibleChildren.length) {
        return context;
    }
    var componentName = utils_1.getComponentName(root, context.filePath);
    context = translateComponent(componentName, target, false, context);
    // KEEP ME: needed for logic
    // if (context.hasLogicFile) {
    //   context = addBuffer(
    //     `${componentName} = enhanceView(${componentName});\n`,
    //     context
    //   );
    // }
    context = translate_utils_1.addBuffer("export default " + componentName + ";\n", context);
    return context;
};
var translateJSXRoot = function (node, context) {
    if (node.kind !== paperclip_1.NodeKind.Fragment) {
        if (node.kind === paperclip_1.NodeKind.Element && node.tagName === paperclip_1.PART_TAG_NAME) {
            return translateFragment(paperclip_1.getVisibleChildNodes(node), true, context);
        }
        else {
            return translateJSXNode(node, true, context);
        }
    }
    var visibleNodes = paperclip_1.getVisibleChildNodes(node);
    if (visibleNodes.length === 1) {
        return translateJSXNode(visibleNodes[0], true, context);
    }
    else {
        context = translateFragment(visibleNodes, true, context);
    }
    return context;
};
var getImportTagName = function (tagName) {
    var parts = tagName.split(".").map(utils_1.pascalCase);
    return parts.length > 1 ? parts[0] + ".default" : parts.join(".");
};
var translateJSXNode = function (node, isRoot, context) {
    if (node.kind === paperclip_1.NodeKind.Fragment) {
        context = translateFragment(node.children, isRoot, context);
    }
    else if (node.kind === paperclip_1.NodeKind.Element && paperclip_1.isVisibleElement(node)) {
        context = translateElement(node, isRoot, context);
    }
    else if (node.kind === paperclip_1.NodeKind.Block) {
        context = translateBlock(node, isRoot, context);
    }
    else if (node.kind === paperclip_1.NodeKind.Text) {
        var buffer = "" + JSON.stringify(node.value);
        if (isRoot) {
            buffer = "React.createElement(\"span\", null, " + buffer + ")";
        }
        context = translate_utils_1.addBuffer(buffer, context);
    }
    else if (node.kind === paperclip_1.NodeKind.Slot) {
        context = translateSlot(node, context);
    }
    return context;
};
var getPartTagName = function (name, context) {
    return utils_1.pascalCase(name);
};
var translateElement = function (element, isRoot, context) {
    var isImportComponentInstance = context.importIds.indexOf(element.tagName) !== -1;
    var isPartComponentInstance = context.partIds.indexOf(element.tagName) !== -1;
    var isComponentInstance = isImportComponentInstance || isPartComponentInstance;
    var id = paperclip_1.getAttributeStringValue("id", element);
    var propsName = null;
    // KEEP ME: for logic
    // const propsName = id
    //   ? `props.${camelCase(id)}Props`
    //   : isComponentInstance
    //   ? `props.${camelCase(element.tagName)}Props`
    //   : null;
    var tag = isPartComponentInstance
        ? getPartTagName(element.tagName, context)
        : isImportComponentInstance
            ? getImportTagName(element.tagName)
            : JSON.stringify(element.tagName);
    context = translate_utils_1.addBuffer("React.createElement(" + tag + ", ", context);
    if (propsName) {
        context = translate_utils_1.addBuffer("extendProps(", context);
    }
    context = translate_utils_1.addBuffer("{\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.startBlock(context);
    context = translateStyleScopeAttributes(context, "\n");
    context = translate_utils_1.addBuffer("\"key\": " + JSON.stringify(String(context.keyCount++)) + (context.currentIndexKey ? " + " + context.currentIndexKey : "") + ",\n", context);
    for (var _i = 0, _a = element.attributes; _i < _a.length; _i++) {
        var attr = _a[_i];
        context = translateAttribute(attr, isComponentInstance, context);
    }
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("}", context);
    if (propsName) {
        context = translate_utils_1.addBuffer(", " + propsName + ")", context);
    }
    context = translate_utils_1.endBlock(context);
    if (element.children.length) {
        context = translate_utils_1.addBuffer(",\n", context);
        context = translateChildren(element.children, false, context);
    }
    else {
        context = translate_utils_1.addBuffer("\n", context);
    }
    context = translate_utils_1.addBuffer(")", context);
    return context;
};
var translateBlock = function (node, isRoot, context) {
    switch (node.blockKind) {
        case paperclip_1.BlockKind.Each:
            return translateEachBlock(node, context);
        case paperclip_1.BlockKind.Conditional:
            return translateConditionalBlock(node, context);
    }
};
var translateEachBlock = function (_a, context) {
    var _b;
    var source = _a.source, body = _a.body, keyName = _a.keyName, valueName = _a.valueName;
    context = translate_utils_1.addBuffer("(", context);
    context = translateStatment(source, false, false, context);
    var key = String(keyName || "$$index" + context.keyCount++);
    context = translate_utils_1.addBuffer(").map(function(" + valueName + ", " + key + ") {\n", context);
    context = translate_utils_1.startBlock(context);
    context = translate_utils_1.addBuffer("return ", context);
    context = translateJSXNode(body, false, __assign(__assign({}, context), { currentIndexKey: key, scopes: __assign(__assign({}, context.scopes), (_b = {}, _b[valueName] = true, _b)) }));
    context = __assign(__assign({}, context), { currentIndexKey: null });
    context = translate_utils_1.addBuffer(";\n", context);
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("})", context);
    return context;
};
var translateConditionalBlock = function (node, context) {
    if (node.conditionalBlockKind === paperclip_1.ConditionalBlockKind.PassFailBlock) {
        context = translate_utils_1.addBuffer("(", context);
        context = translateStatment(node.condition, false, false, context);
        context = translate_utils_1.addBuffer(" ? ", context);
        context = translateJSXNode(node.body, false, context);
        context = translate_utils_1.addBuffer(" : ", context);
        if (node.fail) {
            context = translateConditionalBlock(node.fail, context);
        }
        else {
            context = translate_utils_1.addBuffer("null", context);
        }
        context = translate_utils_1.addBuffer(")", context);
        return context;
    }
    else {
        return translateJSXNode(node.body, false, context);
    }
};
var translateFragment = function (children, isRoot, context) {
    if (children.length === 1) {
        return translateJSXNode(children[0], isRoot, context);
    }
    context = translate_utils_1.addBuffer("[\n", context);
    context = translateChildren(children, false, context);
    context = translate_utils_1.addBuffer("]", context);
    return context;
};
var translateChildren = function (children, isRoot, context) {
    context = translate_utils_1.startBlock(context);
    context = children
        .filter(paperclip_1.isVisibleNode)
        .reduce(function (newContext, child, index, children) {
        newContext = translateJSXNode(child, isRoot, newContext);
        if (index < children.length - 1) {
            newContext = translate_utils_1.addBuffer(",\n", newContext);
        }
        return newContext;
    }, context);
    context = translate_utils_1.endBlock(context);
    if (children.length) {
        context = translate_utils_1.addBuffer("\n", context);
    }
    return context;
};
var isSpecialPropName = function (name) {
    return /^on/.test(name) || name === "checked";
};
var translateAttribute = function (attr, isComponentInstance, context) {
    if (attr.kind === paperclip_1.AttributeKind.KeyValueAttribute) {
        var name_1 = utils_1.RENAME_PROPS[attr.name] || attr.name;
        var value = attr.value;
        if (name_1 === "string") {
            console.warn("Can't handle style tag for now");
        }
        // can't handle for now
        if (name_1 !== "style") {
            if (!/^data-/.test(name_1)) {
                name_1 = lodash_1.camelCase(name_1);
            }
            context = translate_utils_1.addBuffer(JSON.stringify(name_1) + ": ", context);
            context = translateAttributeValue(name_1, value, !isComponentInstance, context);
            context = translate_utils_1.addBuffer(",\n", context);
        }
    }
    else if (attr.kind === paperclip_1.AttributeKind.ShorthandAttribute) {
        var keyValue = attr.reference.path[0];
        var value = "" + (context.scopes[keyValue] ? "" : "props.") + lodash_1.camelCase(keyValue);
        if (!isComponentInstance && !isSpecialPropName(keyValue)) {
            // everything must be a string
            value = value + " ? String(" + value + ") : null";
        }
        context = translate_utils_1.addBuffer(JSON.stringify(keyValue) + ": " + value, context);
        context = translate_utils_1.addBuffer(",\n", context);
    }
    else if (attr.kind === paperclip_1.AttributeKind.SpreadAttribute) {
        context = translate_utils_1.addBuffer("...(", context);
        context = translateStatment(attr.script, false, false, context);
        context = translate_utils_1.addBuffer(")", context);
        context = translate_utils_1.addBuffer(",\n", context);
    }
    return context;
};
var translateAttributeValue = function (name, value, isPropOnNativeElement, context) {
    if (!value) {
        return translate_utils_1.addBuffer("true", context);
    }
    if (value.attrValueKind === paperclip_1.AttributeValueKind.Slot) {
        return translateStatment(value, false, isPropOnNativeElement && !isSpecialPropName(name), context);
    }
    else if (value.attrValueKind === paperclip_1.AttributeValueKind.String) {
        var strValue = JSON.stringify(value.value);
        if (name === "src") {
            strValue = "require(" + strValue + ").default";
        }
        return translate_utils_1.addBuffer(strValue, context);
    }
    return context;
};
var translateSlot = function (slot, context) {
    return translateStatment(slot.script, true, false, context);
};
var translateStatment = function (statement, isRoot, shouldStringifyProp, context) {
    if (statement.jsKind === paperclip_1.StatementKind.Reference) {
        if (shouldStringifyProp) {
            context = translateStatment(statement, isRoot, false, context);
            context = translate_utils_1.addBuffer(" ? String(", context);
        }
        context = translate_utils_1.addBuffer("" + (context.scopes[statement.path[0]] ? "" : "props.") + statement.path.join("."), context);
        if (shouldStringifyProp) {
            context = translate_utils_1.addBuffer(") : null", context);
        }
        return context;
    }
    else if (statement.jsKind === paperclip_1.StatementKind.Node) {
        return translateJSXNode(statement, isRoot, context);
    }
    else if (statement.jsKind === paperclip_1.StatementKind.Array) {
        context = translate_utils_1.addBuffer("[\n", context);
        context = translate_utils_1.startBlock(context);
        for (var _i = 0, _a = statement.values; _i < _a.length; _i++) {
            var value = _a[_i];
            context = translateStatment(value, false, false, context);
            context = translate_utils_1.addBuffer(",\n", context);
        }
        context = translate_utils_1.endBlock(context);
        context = translate_utils_1.addBuffer("]", context);
    }
    else if (statement.jsKind === paperclip_1.StatementKind.Object) {
        context = translate_utils_1.addBuffer("{\n", context);
        context = translate_utils_1.startBlock(context);
        for (var _b = 0, _c = statement.properties; _b < _c.length; _b++) {
            var _d = _c[_b], key = _d.key, value = _d.value;
            context = translate_utils_1.addBuffer(JSON.stringify(key) + ":", context);
            context = translateStatment(value, false, false, context);
            context = translate_utils_1.addBuffer(",\n", context);
        }
        context = translate_utils_1.endBlock(context);
        context = translate_utils_1.addBuffer("}", context);
    }
    else if (statement.jsKind === paperclip_1.StatementKind.Number ||
        statement.jsKind === paperclip_1.StatementKind.String ||
        statement.jsKind === paperclip_1.StatementKind.Boolean) {
        return translate_utils_1.addBuffer(String(statement.value), context);
    }
    return context;
};
