"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var paperclip_1 = require("paperclip");
var translate_utils_1 = require("./translate-utils");
var utils_1 = require("./utils");
exports.compile = function (_a, filePath, options) {
    var ast = _a.ast;
    if (options === void 0) { options = {}; }
    var context = translate_utils_1.createTranslateContext(filePath, paperclip_1.getImportIds(ast), paperclip_1.getPartIds(ast), [], Boolean(paperclip_1.getLogicElement(ast)), options);
    context = translateRoot(ast, context);
    return context.buffer;
};
var translateRoot = function (ast, context) {
    context = translate_utils_1.addBuffer("import {ReactNode, ReactHTML, Factory, InputHTMLAttributes, ClassAttributes} from \"react\";\n\n", context);
    // KEEP ME: logic taken out, but keep if it's re-added
    // later kn
    // const allImports = getImports(ast);
    // for (const imp of allImports) {
    //   const id = getAttributeStringValue("id", imp);
    //   const src = getAttributeStringValue("src", imp);
    //   if (!id || !src) {
    //     continue;
    //   }
    //   const relativePath = getRelativeFilePath(context.filePath, src);
    //   context = addBuffer(
    //     `import {EnhancedProps as ${pascalCase(
    //       getInstancePropsName(imp)
    //     )}} from "${relativePath}";\n`,
    //     context
    //   );
    // }
    // context = addBuffer(`\n`, context);
    // TODO: keep for logic
    // const logicElement = getLogicElement(ast);
    // if (logicElement) {
    //   const src = getAttributeStringValue("src", logicElement);
    //   if (src) {
    //     const logicRelativePath = getRelativeFilePath(context.filePath, src);
    //     context = addBuffer(
    //       `import {Props as LogicProps} from "${logicRelativePath.replace(
    //         /\.tsx?$/,
    //         ""
    //       )}";\n`,
    //       context
    //     );
    //   }
    // }
    context = translate_utils_1.addBuffer("type ElementProps = InputHTMLAttributes<HTMLInputElement> & ClassAttributes<HTMLInputElement>;\n\n", context);
    // KEEP ME: needed for logic
    // context = addBuffer(
    //   `type PropsFactory<TProps> = (props: Partial<TProps>) => TProps;\n\n`,
    //   context
    // );
    context = translateUtils(ast, context);
    context = translateParts(ast, context);
    context = translateMainView(ast, context);
    return context;
};
var translateUtils = function (_ast, context) {
    context = translate_utils_1.addBuffer("export declare const styled: (tag: keyof ReactHTML | Factory<ElementProps>, defaultProps?: ElementProps) => Factory<ElementProps>;\n\n", context);
    return context;
};
var BLACK_LIST_PROPS = {
// className: true,
// children: true
};
var DEFAULT_PARAM_TYPE = "String | boolean | Number | Object | ReactNode";
var translateInference = function (inference, property, context) {
    if (inference.kind === paperclip_1.InferenceKind.Any) {
        return translate_utils_1.addBuffer(/^on\w+/.test(property) ? "Function" : DEFAULT_PARAM_TYPE, context);
    }
    if (inference.kind === paperclip_1.InferenceKind.Array) {
        context = translate_utils_1.addBuffer("Array<", context);
        context = translateInference(inference.value, property, context);
        context = translate_utils_1.addBuffer(">", context);
    }
    if (inference.kind === paperclip_1.InferenceKind.Shape) {
        context = translate_utils_1.addBuffer("{\n", context);
        context = translate_utils_1.startBlock(context);
        for (var key in inference.properties) {
            context = translate_utils_1.addBuffer(key + ": ", context);
            context = translateInference(inference.properties[key], key, context);
            context = translate_utils_1.addBuffer(",\n", context);
        }
        context = translate_utils_1.endBlock(context);
        context = translate_utils_1.addBuffer("}", context);
        if (inference.fromSpread) {
            context = translate_utils_1.addBuffer(" & ElementProps", context);
        }
    }
    return context;
};
var translateComponent = function (node, componentPropsName, context) {
    context = translate_utils_1.addBuffer((context.hasLogicFile ? "export " : "") + "type " + componentPropsName + " = {\n", context);
    context = translate_utils_1.startBlock(context);
    var props = {};
    var inference = paperclip_1.infer(node);
    for (var key in inference.properties) {
        var propName = utils_1.RENAME_PROPS[key] || key;
        if (BLACK_LIST_PROPS[propName]) {
            continue;
        }
        context = translate_utils_1.addBuffer(key + ": ", context);
        context = translateInference(inference.properties[key], key, context);
        context = translate_utils_1.addBuffer(",\n", context);
        props[key] = [null];
    }
    // KEEP ME: needed for logic.
    // const allElements = flattenNodes(node).filter(
    //   node => node.kind === NodeKind.Element && isVisibleElement(node)
    // ) as Element[];
    // for (const element of allElements) {
    //   if (isComponentInstance(element, context.importIds)) {
    //     const valueType = getInstancePropsTypeName(element);
    //     props[getInstancePropsName(element)] = [
    //       `${valueType} | PropsFactory<${valueType}>`,
    //       false
    //     ];
    //   } else {
    //     const id = getAttributeStringValue("id", element);
    //     if (id) {
    //       props[`${camelCase(id)}Props`] = [
    //         `ElementProps | PropsFactory<ElementProps>`
    //       ];
    //     }
    //   }
    // }
    for (var key in props) {
        var _a = props[key], valueType = _a[0], optional = _a[1];
        if (!valueType)
            continue;
        context = translate_utils_1.addBuffer("" + key + (optional ? "?" : "") + ": " + valueType + ",\n", context);
    }
    context = translate_utils_1.endBlock(context);
    context = translate_utils_1.addBuffer("};\n\n", context);
    return context;
};
// const getInstancePropsName = (element: Element) => {
//   return `${camelCase(
//     getAttributeStringValue("id", element) || element.tagName
//   )}Props`;
// };
var getInstancePropsTypeName = function (element) {
    return utils_1.pascalCase(element.tagName) + "Props";
};
var translateParts = function (ast, context) {
    for (var _i = 0, _a = paperclip_1.getParts(ast); _i < _a.length; _i++) {
        var part = _a[_i];
        // already translated, so skip.
        if (paperclip_1.getAttributeStringValue("id", part) === paperclip_1.DEFAULT_PART_ID) {
            continue;
        }
        if (paperclip_1.hasAttribute(paperclip_1.NO_COMPILE_TAG_NAME, part)) {
            continue;
        }
        context = translatePart(part, context);
    }
    return context;
};
var translatePart = function (part, context) {
    var componentName = utils_1.pascalCase(paperclip_1.getAttributeStringValue("id", part));
    var propsName = componentName + "Props";
    context = translateComponent(part, propsName, context);
    context = translate_utils_1.addBuffer("export const " + componentName + ": Factory<" + propsName + ">;\n\n", context);
    return context;
};
var translateMainView = function (ast, context) {
    var target = paperclip_1.getParts(ast).find(function (part) { return paperclip_1.getAttributeStringValue("id", part) === paperclip_1.DEFAULT_PART_ID; }) || ast;
    if (target === ast) {
        var children = paperclip_1.getVisibleChildNodes(ast);
        if (!children.length) {
            return context;
        }
    }
    var componentName = "View";
    context = translateComponent(target, "Props", context);
    context = translate_utils_1.addBuffer("declare const " + componentName + ": Factory<Props>;\n", context);
    // context = addBuffer(`export type EnhancedProps = Props;\n`, context);
    context = translate_utils_1.addBuffer("export default " + componentName + ";\n", context);
    // KEEP ME: logic has been taken out, but may
    // be re-added later on.
    // if (context.hasLogicFile) {
    //   context = addBuffer(
    //     `declare const Enhanced${componentName}: Factory<LogicProps>;\n`,
    //     context
    //   );
    //   context = addBuffer(`export type EnhancedProps = LogicProps;\n`, context);
    //   context = addBuffer(`export default Enhanced${componentName};\n`, context);
    // } else {
    //   context = addBuffer(`export type EnhancedProps = Props;\n`, context);
    //   context = addBuffer(`export default ${componentName};\n`, context);
    // }
    return context;
};
